import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  OneToMany,
  ManyToOne,
  Timestamp,
} from 'typeorm';
import { GameEntity } from './game.entity';
import { Game_PersonEntity } from './game.entity';
import { MapDrawingEntity } from './coordinate.entity';

//Faction, PowerUp, Faction_powerUp, FP_History, Score, Task

@Entity('Faction')
export class FactionEntity {
  @PrimaryGeneratedColumn('uuid') factionId: string;
  @Column('text') factionName: string;
  @Column({ type: 'text' }) factionPassword: string;
  @Column({ type: 'float' }) multiplier: number;

  @ManyToOne(type => GameEntity, gameEntity => gameEntity.id)
  gameId: GameEntity;
}

@Entity('PowerUp')
export class PowerUpEntity {
  @PrimaryGeneratedColumn('uuid') powerUpId: string;
  @Column({ type: 'text' }) powerUpName: string;
  @Column({ type: 'text' }) powerUpDescription: string;
  @Column({ type: 'int' }) amount: number;
  @Column({ type: 'time' }) cooldown: string;
}

@Entity('Faction_PowerUp')
export class Faction_PowerUpEntity {
  @PrimaryGeneratedColumn('uuid') faction_powerUpId: string;

  @ManyToOne(type => FactionEntity, faction => faction.factionId)
  faction: FactionEntity;
  @ManyToOne(type => PowerUpEntity, powerUp => powerUp.powerUpId)
  powerUp: PowerUpEntity;
}

@Entity('FP_History')
export class FP_HistoryEntity {
  @PrimaryGeneratedColumn('uuid') historyId: string;
  @Column({ type: 'timestamp' }) historyTimeStamp: Timestamp;

  @ManyToOne(
    type => Faction_PowerUpEntity,
    faction_PowerUp => faction_PowerUp.faction_powerUpId,
  )
  faction_PowerUp: Faction_PowerUpEntity;
}

@Entity('Score')
export class ScoreEntity {
  @PrimaryGeneratedColumn('uuid') scoreId: string;
  @Column({ type: 'float' }) score: number;
  @Column({ type: 'timestamp' }) scoreTimeStamp: Timestamp;

  @ManyToOne(type => FactionEntity, factionName => factionName.factionId)
  faction: FactionEntity;
}

@Entity('Task')
export class TaskEntity {
  @PrimaryGeneratedColumn('uuid') taskId: string;
  @Column({ type: 'text' }) taskName: string;
  @Column({ type: 'text' }) taskDescription: string;
  @Column({ type: 'text' }) taskWinner: string;
  @Column({ type: 'bool' }) taskIsActive: boolean;

  @ManyToOne(type => FactionEntity, faction => faction.factionId)
  faction: FactionEntity;
  @ManyToOne(type => Game_PersonEntity, person => person.gamepersonId)
  personId: Game_PersonEntity;
}
